import sys
import copy
from JGCalendar import *
#■固定休日テーブル
#
#  各要素は[N, 月, 日, 休日名, 元の日付, 有効年開始,有効年終了]の形式
#
#    N =  0  : 固定
#    N <  5  : その月の第N週の月曜日を休日とする
#    N = 10  : 春分の日/春季皇霊祭(計算による)
#    N = 11  : 秋分の日/秋季皇霊祭(計算による)
#
#    元の日付: 振替休日のときの元の日付
#              元々の休日のとき最大日
#              (第N週月曜日のとき N × 7, 固定のとき日と同じ)
#
#  (注)途中で国民の休日の定義が変わった日は、ふたつ以上の定義を用意し、
#      リストに取り出すとき、弾かれます。
#
Ftab=[[ 0,  1,  1,"四方節"          ,  0, 1873, 1947],
      [ 0,  1,  1,"元旦"            ,  1, 1948,      999999],
      [ 0,  1,  3,"元始祭"          ,  3, 1878, 1947],
      [ 0,  1,  5,"新年宴会"        ,  5, 1878, 1947],
      [ 0,  1, 15,"成人の日"        , 15, 1948, 1999],
      [ 2,  1, 14,"成人の日"        , 14, 2000,      999999],
      [ 0,  1, 30,"孝明天皇祭"      , 30, 1878, 1947],
      [ 0,  2, 11,"紀元節"          , 11, 1872, 1948],
      [ 0,  2, 11,"建国記念日"      , 11, 1967,      999999],
      [ 0,  2, 23,"天皇誕生日"      , 23, 2020,      999999],
      [10,  3, 20,"春季皇霊祭"      , 20, 1878, 1947],
      [10,  3, 20,"春分の日"        , 20, 1948,      999999],
      [ 0,  4,  3,"神武天皇祭"      ,  3, 1878, 1947],
      [ 0,  4, 29,"天長節(昭和)"    , 29, 1927, 1947],
      [ 0,  4, 29,"天皇誕生日(昭和)", 29, 1948, 1988],
      [ 0,  4, 29,"みどりの日"      , 29, 1989, 2006],
      [ 0,  4, 29,"昭和の日"        , 29, 2007,     999999],
      [ 0,  5,  3,"憲法記念日"      ,  3, 1948,     999999],
      [ 0,  5,  4,"みどりの日"      ,  4, 2007,     999999],
      [ 0,  5,  5,"こどもの日"      ,  5, 1948,     999999],
      [ 0,  7, 20,"海の日"          , 20, 1996, 2002],
      [ 3,  7, 21,"海の日"          , 21, 2003, 2019],
      [ 3,  7, 21,"海の日"          , 21, 2021,     999999],
      [ 0,  7, 23,"海の日"          , 23, 2020, 2020],#特例
      [ 0,  7, 24,"スポーツの日"    , 24, 2020, 2020],#特例
      [ 0,  8, 10,"山の日"          , 10, 2020, 2020],#特例
      [ 0,  8, 11,"山の日"          , 11, 2016, 2019],
      [ 0,  8, 11,"山の日"          , 11, 2021,     999999],
      [ 0,  8, 31,"天長節(大正)"    , 31, 1913, 1926],
      [ 0,  9, 15,"敬老の日"        , 15, 1966, 2002],
      [ 3,  9, 21,"敬老の日"        , 21, 2003,     999999],
      [ 0,  9, 17,"神嘗祭"          , 17, 1874, 1878],
      [11,  9, 23,"秋季皇霊祭"      , 23, 1878, 1947],
      [11,  9, 23,"秋分の日"        , 23, 1948,     999999],
      [ 0, 10, 10,"体育の日"        , 10, 1966, 1999],
      [ 2, 10, 14,"体育の日"        , 14, 2000, 2019],
      [ 2, 10, 14,"スポーツの日"    , 14, 2021,     999999],
      [ 0, 10, 17,"神嘗祭"          , 17, 1878, 1947],
      [ 0, 10, 31,"天長節祝日(大正)", 31, 1913, 1926],
      [ 0, 11,  3,"天長節(明治)"    ,  3, 1866, 1911],
      [ 0, 11,  3,"明治節"          ,  3, 1927, 1947],
      [ 0, 11,  3,"文化の日"        ,  3, 1948,     999999],
      [ 0, 11, 23,"新嘗祭"          , 23, 1878, 1947],
      [ 0, 11, 23,"勤労感謝の日"    , 23, 1948,     999999],
      [ 0, 12, 23,"天皇誕生日(平成)", 23, 1989, 2018]]

# 春分の日のときの計算用係数(先頭2個が年の範囲を示す)
SpTab=((1851, 1899, 19.8277, 0.242194, 1980, 1983),
       (1900, 1979, 20.8357, 0.242194, 1980, 1983),
       (1980, 2099, 20.8431, 0.242194, 1980, 1980),
       (2100, 2150, 21.8510, 0.242194, 1980, 1980))
# 秋分日のときの計算用係数
AuTab=((1851, 1899, 22.2588, 0.242194, 1980, 1983),
       (1900, 1979, 23.2588, 0.242194, 1980, 1983),
       (1980, 2099, 23.2488, 0.242194, 1980, 1980),
       (2100, 2150, 24.2488, 0.242194, 1980, 1980))
#■春分・秋分の日の計算(春分/秋分はテーブルで指定)
def getSpAu(Y, Tab):
    for i in range(len(Tab)):
        if Y>=Tab[i][0] and Y<=Tab[i][1]:
            return int(Tab[i][2]+Tab[i][3]*(Y-Tab[i][4])-
                       int((Y-Tab[i][5])/4))
    return -1

#■月曜日を 0 とする曜日の番号(土曜日は 7 )
def startMonday(Y,M,D):
    N=GWeekDayZ([Y,M,D])-1;
    if N<0: N=6
    return N
#■第N週
def getMonHoliday(Y,M,D, N):
    return 7*N-startMonday(Y,M,D)
#■Ftabに入っているかどうかのチェック
def inFtab(DT):
    for i in range(len(Ftab)):
        if DT[1]==Ftab[i][1] and DT[2]==Ftab[i][2]: return True
    return False
#■休日リストの日付設定
def getFixHolidays(Y):
    for i in range(len(Ftab)):
        if Ftab[i][0]>0<5 and Ftab[i][0]<5:
            Ftab[i][2]=getMonHoliday(Y,Ftab[i][1],Ftab[i][4],Ftab[i][0])
        elif Ftab[i][0]==10:Ftab[i][2]=getSpAu(Y,SpTab)
        elif Ftab[i][0]==11:Ftab[i][2]=getSpAu(Y,AuTab)
#■有効年だけを取り出す
def copyHolidays(Y):
    R=[]
    for i in range(len(Ftab)):
        if Y>=Ftab[i][5] and Y<=Ftab[i][6]: R.append(Ftab[i])
    return R
#■CHDTに入っているかどうかのチェック
def inCHDT(DT,N):
    for i in range(N):
        if(DT[0]>=Ftab[i][5] and DT[0]<=Ftab[i][6] and
           DT[1]==Ftab[i][1] and DT[2]==Ftab[i][2]): return True
    return False
#■次の日
def nextDay(Date):
    R=[0,1,1]
    if Date[1]==12 and Date[2]==31: R[0]=Date[0]+1;return R
    R[0]=Date[0]
    if Date[2]==GMDates(Date):R[1]=Date[1]+1; return R
    R[1]=Date[1]; R[2]=Date[2]+1; return R
#■前日
def beforDay(Date):
    R=[0,12,31]
    if Date[1]==1 and Date[2]==1: R[0]=Date[0]-1;return R
    R[0]=Date[0]
    if Date[2]==1:R[1]=Date[1]-1; R[2]=GMDates(R);return R
    R[1]=Date[1]; R[2]=Date[2]-1; return R

#■振替休日の追加
#  固定休日テーブルに追加すると副作用がおきるので
#  予め複写したテーブルに追加する。
def changeHolidays(Y):
    DT=[0,0,0]; DT[0]=Y; addDT=[ -1, 0, 0,"振替休日",0, 0, 999999];
    CHDT=copyHolidays(Y); N=len(CHDT)
    for i in range(N):
        DT[1]=CHDT[i][1]; DT[2]=CHDT[i][2]; DT=nextDay(DT)
        if Y>=2007: #■2007年以降
            if GWeekDayZ([Y, CHDT[i][1],CHDT[i][2]])==0:
                while inCHDT(DT,N): DT=nextDay(DT)
                addDT[1]=DT[1]#国民の祝日に関する法律 第三条 第２項
                addDT[2]=DT[2]#国民の祝日が日曜のとき、その日後において
                addDT[4]=CHDT[i][2] #最も近い国民の祝日でない日を休日とする。
                CHDT.append(copy.copy(addDT))
            else:
                if (not(inCHDT(DT,N)) and inCHDT(nextDay(DT),N) and
                    inCHDT(beforDay(DT),N)):
                    addDT[1]=DT[1]# 国民の祝日に関する法律 第三条 第３項
                    addDT[2]=DT[2]# その前日及び翌日が国民の祝日のとき
                    addDT[4]==CHDT[i][2] #（その日が祝日でない）休日とする
                    CHDT.append(copy.copy(addDT))
        elif Y>=1973:#1973～2006年：日曜が休日のとき次の月曜を振替休日
            if GWeekDayZ([Y, CHDT[i][1],CHDT[i][2]])==0:
                addDT[1]=DT[1]
                addDT[2]=DT[2]
                addDT[4]=CHDT[i][2] 
                CHDT.append(copy.copy(addDT))
    return CHDT
#■休日リストの作成
def getHolidays(Y):
    #if Y<1873: return []
    getFixHolidays(Y);  return changeHolidays(Y)
#■休日リストの表示
def printHolidays(Y, P):
    Week="日月火水木金土"; N=len(P)
    for i in range(N):
        WD=Week[GWeekDayZ([Y, P[i][1],P[i][2]])];  AD=""
        if P[i][0]<0:
            for k in range(N):
                if P[k][1]==P[i][1] and P[k][2]==P[i][4]: AD="["+P[k][3]+"]"; break
        print("%2d/%2d(%s) %s %s" % (P[i][1],P[i][2],WD,P[i][3],AD))
#■休日リストから指定月だけを取り出す
def selectMounth(P,M):
    R=[]
    for i in range(len(P)):
        if P[i][1]==M: R.append(P[i])
    return R
#■休日リストにあるかどうか
def isHoliday(Holydays, M,D):
    
    for i in range(len(Holydays)):
        if Holydays[i][1]==M and Holydays[i][2]==D: return i
    return -1
#■テスト用プログラム
def testHolidays():
    while 1:
        s=sys.stdin.readline()
        if(s=="\n"): break
        Y=int(s); L=getHolidays(Y); printHolidays(Y, L)
        print("\n")
        
 
